@@ -39,8 +39,8 @@ from .utils.etl import log_power_transform, decompose, normalize, split_data
 
 
 def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
-                  time_interval='', forecast_period='', auto_clean=False, epochs=10,
-                  batch_size=256, score_type='rmse', non_seasonal_pdq=None,
+                  time_interval='', forecast_period='', auto_load=True, auto_clean=False,
+                  epochs=10, batch_size=256, score_type='rmse', non_seasonal_pdq=None,
                   seasonality=False, seasonal_period=12, seasonal_PDQ=None,
                   conf_int=0.95, model_type="stats", verbose=0):
     """
@@ -126,15 +126,20 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
     print("-------------------------------------------------------------------\n")
 
     # logger.info("Test123.")
-    print("Begin extracting the data from source...")
-    ts_df = load_ts_data(data, ts_column, sep)
-    new_key = f'{data_counter}) Original'
-    data_dict[new_key] = ts_df
-    print(f"\n--> Extracted data saved in data_dict[{new_key}], see head below:")
-    display(HTML(ts_df.head().to_html()))
-    data_counter += 1
+    if auto_load:
+        print("Begin extracting the data from source...")
+        ts_df = load_ts_data(data, ts_column, sep)
+        new_key = f'{data_counter}) Original'
+        data_dict[new_key] = ts_df
+        print(f"\n--> Extracted data saved in data_dict[{new_key}], see head below:")
+        display(HTML(ts_df.head().to_html()))
+        data_counter += 1
+    else:
+        print("Auto-load has been skipped.")
+        ts_df = copy.deepcopy(data)
 
     freq, _ = interval_to_freq(time_interval)
+    print(f"Frequency has been set to {freq}.")
 
     ########## This is where we start the loading of the data file ######################
     # if isinstance(trainfile, str):
@@ -151,19 +156,20 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
     # else:
     #     print('File name is an empty string. Please check your input and try again')
     #     return
-    # df_orig = copy.deepcopy(ts_df)
-    # if ts_df.shape[1] == 1:
-    #     ### If there is only one column, you assume that to be the target column ####
-    #     target = list(ts_df)[0]
-    # if not isinstance(ts_column, str):
-    #     ### If ts_column is a number, then it means you need to convert it to a named variable
-    #     ts_column = list(ts_df)[ts_column]
-    # if isinstance(target,list):
-    #     target = target[0]
-    #     print('    Taking the first column in target list as Target variable = %s' %target)
-    # else:
-    #     print('    Target variable = %s' %target)
-    # preds = [x for x in list(ts_df) if x not in [ts_column, target]]
+    df_orig = copy.deepcopy(ts_df)
+    if ts_df.shape[1] == 1:
+        ### If there is only one column, you assume that to be the target column ####
+        target = list(ts_df)[0]
+    if not isinstance(ts_column, str):
+        ### If ts_column is a number, then it means you need to convert it to a named variable
+        ts_column = list(ts_df)[ts_column]
+    if isinstance(target,list):
+        target = target[0]
+        print('    Taking the first column in target list as Target variable = %s' %target)
+    else:
+        print('    Target variable = %s' %target)
+    preds = [x for x in list(ts_df) if x not in [ts_column, target]]
+    # ipdb.set_trace()
 
 
     print("\n\n-------------------------------------------------------------------")
@@ -178,6 +184,9 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
         print(f"\n--> Cleaned data saved in data_dict[{new_key}], see head below:")
         display(HTML(ts_df.head().to_html()))
         data_counter += 1
+    else:
+        print("Auto-clean has been skipped.")
+
 
     print("\n\n-------------------------------------------------------------------")
     print("STEP 3) EDA I (General)")
@@ -192,10 +201,10 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
     x_label = f'Datetime'
     y_label = f"Total Demand (MW)"
 
-    ts_plot(ts_df, ts_column, target,
-            title=title,
-            y_label=y_label
-            )
+    # ts_plot(ts_df, ts_column, target,
+    #         title=title,
+    #         y_label=y_label
+    #         )
 
     print("\n\n-------------------------------------------------------------------")
     print("STEP 4) EDA II (Time-Series Stats)")
@@ -209,15 +218,15 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
     # ets = ets_decomposition_plot(ts_df, ts_column, target, title, y_label);
     # ets = ets_decomposition_plot(ts_df, ts_column, target, title, y_label,
     #                        prophet=True);
-    ets = ets_decomposition_plot(ts_df, ts_column, target, title, y_label,
-                                 plotly=True);
+    # ets = ets_decomposition_plot(ts_df, ts_column, target, title, y_label,
+    #                              plotly=True);
 
     print("Plot out ACF/PACF graph..\n")
     title = "Total Electricity Demand"
     lags_7 = 24*7  # 7-days lag
     lags_30 = 24*30  # 30-days lag
     lags_90 = 24*90  # 90-days lag
-    acf_pacf_plot(ts_df, target, title, lags=[lags_7, lags_30])
+    # acf_pacf_plot(ts_df, target, title, lags=[lags_7, lags_30])
 
     print("4.4) Expotential Smoothing Holt-Winters.\n")
 
@@ -249,27 +258,27 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
     print("-------------------------------------------------------------------\n")
 
     # Performs log or power transform and then normalize in one function
-    if True:
-        print(f"5.1) Performed power transformation using Box-Cox method. Then standardized data.")
-    else:
-        print(f"5.1) Performed power transformation using Box-Cox method.")
-    ts_df, trans_type = log_power_transform(ts_df, target, method='box-cox',
-                                            standardize=True)
-    new_key = f'{data_counter}) {trans_type}'
-    data_dict[new_key] = ts_df
-    print(f"\n--> Transformed data saved in data_dict[{new_key}], see head below:")
-    display(HTML(ts_df.head().to_html()))
-    data_counter += 1
-
-    # Performs log or power transform and then normalize in one function
-    print(f"\n5.2) Performed seaonsal adjustment.")
-    ts_df, seasonality = decompose(ts_df, target, type='deseasonalize',
-                                   model='additive')
-    new_key = f'{data_counter}) Deseasonalize'
-    data_dict[new_key] = ts_df
-    print(f"\n--> Seasonality adjusted data saved in data_dict[{new_key}], see head below:")
-    display(HTML(ts_df.head().to_html()))
-    data_counter += 1
+    # if True:
+    #     print(f"5.1) Performed power transformation using Box-Cox method. Then standardized data.")
+    # else:
+    #     print(f"5.1) Performed power transformation using Box-Cox method.")
+    # ts_df, trans_type = log_power_transform(ts_df, target, method='box-cox',
+    #                                         standardize=True)
+    # new_key = f'{data_counter}) {trans_type}'
+    # data_dict[new_key] = ts_df
+    # print(f"\n--> Transformed data saved in data_dict[{new_key}], see head below:")
+    # display(HTML(ts_df.head().to_html()))
+    # data_counter += 1
+    #
+    # # Performs log or power transform and then normalize in one function
+    # print(f"\n5.2) Performed seaonsal adjustment.")
+    # ts_df, seasonality = decompose(ts_df, target, type='deseasonalize',
+    #                                model='additive')
+    # new_key = f'{data_counter}) Deseasonalize'
+    # data_dict[new_key] = ts_df
+    # print(f"\n--> Seasonality adjusted data saved in data_dict[{new_key}], see head below:")
+    # display(HTML(ts_df.head().to_html()))
+    # data_counter += 1
 
 
     ##################################################################################################
@@ -333,18 +342,165 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
     #     print('Time Interval is given as %s' % timeinterval)
 
 
+    ########################### This is where we store all models in a nested dictionary ##########
+    mldict = lambda: defaultdict(mldict)
+    ml_dict = mldict()
+    try:
+        if model_type.lower() == 'all':
+            print('Running all model types. This will take a long time. Be Patient...')
+    except:
+        print('Check if your model type is a string or one of the available types of models')
+
+    print("\n\n-------------------------------------------------------------------")
+    print("STEP 6) Model Search I (Naive Models)")
+    print("-------------------------------------------------------------------\n")
+
+
+    print("\n\n-------------------------------------------------------------------")
+    print("STEP 7) Model Search II (Stat Models)")
+    print("-------------------------------------------------------------------\n")
+
+    ### If run_prophet is set to True, then only 1 model will be run and that is FB Prophet ##
+    if isinstance(forecast_period, str):
+        forecast_period = interval_to_timesteps(forecast_period, freq)
+        print(f"Converted forecast_period into num of timesteps of {forecast_period}.")
+    lag = copy.deepcopy(forecast_period)-1
+    if type(non_seasonal_pdq) == tuple:
+        p_max = non_seasonal_pdq[0]
+        d_max = non_seasonal_pdq[1]
+        q_max = non_seasonal_pdq[2]
+    else:
+        p_max = 12
+        d_max = 1
+        q_max = 12
+    ################################
+    if type(seasonal_PDQ) == tuple:
+        seasonal_order = copy.deepcopy(seasonal_PDQ)
+    else:
+        seasonal_order = (3, 1, 3)
+
+    ######### This is when you need to use FB Prophet ###################################
+    ### When the time interval given does not match the tested_timeinterval, then use FB.
+    #### Also when the number of rows in data set is very large, use FB Prophet, It is fast.
+    #########                 FB Prophet              ###################################
+    if model_type.lower() in ['prophet','all']:
+        name = 'FB_Prophet'
+        print(colorful.BOLD + '\nRunning Facebook Prophet Model...' + colorful.END)
+        # try:
+        #### If FB prophet needs to run, it needs to be installed. Check it here ###
+        model, forecast_df, rmse, norm_rmse = build_prophet_model(
+                                    ts_df, ts_column, target, forecast_period,
+                                    score_type, verbose, conf_int)
+        ml_dict[name]['model'] = model
+        ml_dict[name]['forecast'] = forecast_df['yhat'].values
+        ##### Make sure that RMSE works, if not set it to np.inf  #########
+        if score_type == 'rmse':
+            score_val = rmse
+        else:
+            score_val = norm_rmse
+        # except:
+        #     print('    FB Prophet may not be installed or Model is not running...')
+        #     score_val = np.inf
+        ml_dict[name][score_type] = score_val
+    if model_type.lower() in ['stats', 'all']:
+        ##### First let's try the following models in sequence #########################################
+        # nsims = 100   ### this is needed only for M-H models in PyFlux
+        # name = 'PyFlux'
+        # print(colorful.BOLD + '\nRunning PyFlux Model...' + colorful.END)
+        # try:
+        #     ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = \
+        #         build_pyflux_model(ts_df, target, p_max, q_max, d_max, forecast_period,
+        #                            'MLE', nsims, score_type, verbose)
+        #     if isinstance(rmse, str):
+        #         print('    PyFlux not installed. Install PyFlux and run it again')
+        #         score_val = np.inf
+        #         rmse = np.inf
+        #         norm_rmse = np.inf
+        # except:
+        #     print('    PyFlux model error: predictions not available.')
+        #     score_val = np.inf
+        #     rmse = np.inf
+        #     norm_rmse = np.inf
+        # ##### Make sure that RMSE works, if not set it to np.inf  #########
+        # if score_type == 'rmse':
+        #     score_val = rmse
+        # else:
+        #     score_val = norm_rmse
+        # ml_dict[name][score_type] = score_val
+        ################### Let's build an ARIMA Model and add results #################
+        name = 'ARIMA'
+        print(colorful.BOLD + '\nRunning Non Seasonal ARIMA Model...' + colorful.END)
+        try:
+            ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = build_arima_model(ts_df[target],
+                                                    stats_scoring,p_max,d_max,q_max,
+                                    forecast_period=forecast_period,method='mle',verbose=verbose)
+        except:
+            print('    ARIMA model error: predictions not available.')
+            score_val = np.inf
+        if score_type == 'rmse':
+            score_val = rmse
+        else:
+            score_val = norm_rmse
+        ml_dict[name][score_type] = score_val
+        ############# Let's build a SARIMAX Model and get results ########################
+        name = 'SARIMAX'
+        print(colorful.BOLD + '\nRunning Seasonal SARIMAX Model...' + colorful.END)
+        # try:
+        ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = build_sarimax_model(ts_df[target], stats_scoring, seasonality,
+                                                seasonal_period, p_max, d_max, q_max,
+                                                forecast_period,verbose)
+        # except:
+        #     print('    SARIMAX model error: predictions not available.')
+        #     score_val = np.inf
+        if score_type == 'rmse':
+            score_val = rmse
+        else:
+            score_val = norm_rmse
+        ml_dict[name][score_type] = score_val
+        ########### Let's build a VAR Model - but first we have to shift the predictor vars ####
+        name = 'VAR'
+        if len(preds) == 0 or univarate:
+            print('No VAR model since number of predictors is zero')
+            rmse = np.inf
+            norm_rmse = np.inf
+        else:
+            try:
+                if df_orig.shape[1] > 1:
+                    preds = [x for x in list(df_orig) if x not in [target]]
+                    print(colorful.BOLD + '\nRunning VAR Model...' + colorful.END)
+                    print('    Shifting %d predictors by 1 to align prior predictor values with current target values...'
+                                            %len(preds))
+                    ts_df[preds] = ts_df[preds].shift(1)
+                    ts_df.dropna(axis=0,inplace=True)
+                    ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = build_var_model(ts_df[[target]+preds],stats_scoring,
+                                                forecast_period, p_max, q_max)
+                else:
+                    print(colorful.BOLD + '\nNo predictors available. Skipping VAR model...' + colorful.END)
+                    score_val = np.inf
+            except:
+                print('    VAR model error: predictions not available.')
+                rmse = np.inf
+                norm_rmse = np.inf
+        ################################################################
+        if score_type == 'rmse':
+            score_val = rmse
+        else:
+            score_val = norm_rmse
+        ########################################################################
+        ml_dict[name][score_type] = score_val
+
+
     ##################################################################################################
     ### Transform dataset into supervised ML problem with walk-forward validation.
     ### Shifting time-steps
     ##################################################################################################
     print("\n\n-------------------------------------------------------------------")
-    print("STEP 6) Preprocessing III (Make Supervised)")
+    print("STEP 8) Preprocessing III (Make Supervised)")
     print("-------------------------------------------------------------------\n")
 
-    ################# This is where you test the data and find the time interval #######
-    #create train, test data
+    ################ This is where you test the data and find the time interval #######
     n_input = interval_to_timesteps('biweek', freq)  # let's input the two weeks (24*14 timesteps)
-    n_output = interval_to_timesteps(forecast_period, freq)  # let's forecast the next day (24 timesteps)
+    n_output = forecast_period  # let's forecast the next day (24 timesteps)
     n_val = interval_to_timesteps('biweek', freq)  # validation dataset size
     n_test = interval_to_timesteps('month', freq)
 
@@ -419,151 +575,6 @@ def make_ts_magic(data, ts_column, sep=',', target=None, univarate=True,
     print(f"\n--> Supervised data saved in data_dict[{new_key}].")
     data_counter += 1
 
-    ########################### This is where we store all models in a nested dictionary ##########
-    mldict = lambda: defaultdict(mldict)
-    ml_dict = mldict()
-    try:
-        if model_type.lower() == 'all':
-            print('Running all model types. This will take a long time. Be Patient...')
-    except:
-        print('Check if your model type is a string or one of the available types of models')
-
-    print("\n\n-------------------------------------------------------------------")
-    print("STEP 7) Model Search I (Naive Models)")
-    print("-------------------------------------------------------------------\n")
-
-
-    print("\n\n-------------------------------------------------------------------")
-    print("STEP 8) Model Search II (Stat Models)")
-    print("-------------------------------------------------------------------\n")
-
-    ### If run_prophet is set to True, then only 1 model will be run and that is FB Prophet ##
-    forecast_period = n_output  # renamed forecast_period to n_output
-    lag = copy.deepcopy(forecast_period)-1
-    if type(non_seasonal_pdq) == tuple:
-        p_max = non_seasonal_pdq[0]
-        d_max = non_seasonal_pdq[1]
-        q_max = non_seasonal_pdq[2]
-    else:
-        p_max = 12
-        d_max = 1
-        q_max = 12
-    ################################
-    if type(seasonal_PDQ) == tuple:
-        seasonal_order = copy.deepcopy(seasonal_PDQ)
-    else:
-        seasonal_order = (3, 1, 3)
-
-    ######### This is when you need to use FB Prophet ###################################
-    ### When the time interval given does not match the tested_timeinterval, then use FB.
-    #### Also when the number of rows in data set is very large, use FB Prophet, It is fast.
-    #########                 FB Prophet              ###################################
-    if model_type.lower() in ['prophet','all']:
-        name = 'FB_Prophet'
-        print(colorful.BOLD + '\nRunning Facebook Prophet Model...' + colorful.END)
-        # try:
-        #### If FB prophet needs to run, it needs to be installed. Check it here ###
-        model, forecast_df, rmse, norm_rmse = build_prophet_model(
-                                    ts_df, ts_column, target, forecast_period,
-                                    score_type, verbose, conf_int)
-        ml_dict[name]['model'] = model
-        ml_dict[name]['forecast'] = forecast_df['yhat'].values
-        ##### Make sure that RMSE works, if not set it to np.inf  #########
-        if score_type == 'rmse':
-            score_val = rmse
-        else:
-            score_val = norm_rmse
-        # except:
-        #     print('    FB Prophet may not be installed or Model is not running...')
-        #     score_val = np.inf
-        ml_dict[name][score_type] = score_val
-    if model_type.lower() in ['stats','all']:
-        ##### First let's try the following models in sequence #########################################
-        nsims = 100   ### this is needed only for M-H models in PyFlux
-        name = 'PyFlux'
-        print(colorful.BOLD + '\nRunning PyFlux Model...' + colorful.END)
-        try:
-            ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = \
-                build_pyflux_model(ts_df, target, p_max, q_max, d_max, forecast_period,
-                                   'MLE', nsims, score_type, verbose)
-            if isinstance(rmse,str):
-                print('    PyFlux not installed. Install PyFlux and run it again')
-                score_val = np.inf
-                rmse = np.inf
-                norm_rmse = np.inf
-        except:
-            print('    PyFlux model error: predictions not available.')
-            score_val = np.inf
-            rmse = np.inf
-            norm_rmse = np.inf
-        ##### Make sure that RMSE works, if not set it to np.inf  #########
-        if score_type == 'rmse':
-            score_val = rmse
-        else:
-            score_val = norm_rmse
-        ml_dict[name][score_type] = score_val
-        ################### Let's build an ARIMA Model and add results #################
-        name = 'ARIMA'
-        print(colorful.BOLD + '\nRunning Non Seasonal ARIMA Model...' + colorful.END)
-        try:
-            ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = build_arima_model(ts_df[target],
-                                                    stats_scoring,p_max,d_max,q_max,
-                                    forecast_period=forecast_period,method='mle',verbose=verbose)
-        except:
-            print('    ARIMA model error: predictions not available.')
-            score_val = np.inf
-        if score_type == 'rmse':
-            score_val = rmse
-        else:
-            score_val = norm_rmse
-        ml_dict[name][score_type] = score_val
-        ############# Let's build a SARIMAX Model and get results ########################
-        name = 'SARIMAX'
-        print(colorful.BOLD + '\nRunning Seasonal SARIMAX Model...' + colorful.END)
-        # try:
-        ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = build_sarimax_model(ts_df[target], stats_scoring, seasonality,
-                                                seasonal_period, p_max, d_max, q_max,
-                                                forecast_period,verbose)
-        # except:
-        #     print('    SARIMAX model error: predictions not available.')
-        #     score_val = np.inf
-        if score_type == 'rmse':
-            score_val = rmse
-        else:
-            score_val = norm_rmse
-        ml_dict[name][score_type] = score_val
-        ########### Let's build a VAR Model - but first we have to shift the predictor vars ####
-        name = 'VAR'
-        if len(preds) == 0:
-            print('No VAR model since number of predictors is zero')
-            rmse = np.inf
-            norm_rmse = np.inf
-        else:
-            try:
-                if df_orig.shape[1] > 1:
-                    preds = [x for x in list(df_orig) if x not in [target]]
-                    print(colorful.BOLD + '\nRunning VAR Model...' + colorful.END)
-                    print('    Shifting %d predictors by 1 to align prior predictor values with current target values...'
-                                            %len(preds))
-                    ts_df[preds] = ts_df[preds].shift(1)
-                    ts_df.dropna(axis=0,inplace=True)
-                    ml_dict[name]['model'], ml_dict[name]['forecast'], rmse, norm_rmse = build_var_model(ts_df[[target]+preds],stats_scoring,
-                                                forecast_period, p_max, q_max)
-                else:
-                    print(colorful.BOLD + '\nNo predictors available. Skipping VAR model...' + colorful.END)
-                    score_val = np.inf
-            except:
-                print('    VAR model error: predictions not available.')
-                rmse = np.inf
-                norm_rmse = np.inf
-        ################################################################
-        if score_type == 'rmse':
-            score_val = rmse
-        else:
-            score_val = norm_rmse
-        ########################################################################
-        ml_dict[name][score_type] = score_val
-
 
     print("\n\n-------------------------------------------------------------------")
     print("STEP 9) Model Search III (ML Models)")
